package ru.dbotthepony.mc.otm.datagen.loot

import net.minecraft.advancements.critereon.StatePropertiesPredicate
import net.minecraft.data.loot.BlockLoot
import net.minecraft.world.item.DyeColor
import net.minecraft.world.item.ItemStack
import net.minecraft.world.item.Items
import net.minecraft.world.level.block.state.properties.BlockStateProperties
import net.minecraft.world.level.block.state.properties.DoubleBlockHalf
import net.minecraft.world.level.storage.loot.LootContext
import net.minecraft.world.level.storage.loot.entries.LootItem
import net.minecraft.world.level.storage.loot.entries.LootPoolEntries
import net.minecraft.world.level.storage.loot.parameters.LootContextParamSets
import net.minecraft.world.level.storage.loot.predicates.LootItemBlockStatePropertyCondition
import ru.dbotthepony.mc.otm.block.entity.ChemicalGeneratorBlockEntity
import ru.dbotthepony.mc.otm.block.entity.EnergyCounterBlockEntity
import ru.dbotthepony.mc.otm.block.entity.MatteryBlockEntity.Companion.ENERGY_KEY
import ru.dbotthepony.mc.otm.block.entity.MatteryBlockEntity.Companion.MATTER_STORAGE_KEY
import ru.dbotthepony.mc.otm.block.entity.MatteryBlockEntity.Companion.REDSTONE_SETTING_KEY
import ru.dbotthepony.mc.otm.block.entity.MatteryBlockEntity.Companion.REDSTONE_SIGNAL_KEY
import ru.dbotthepony.mc.otm.block.entity.matter.MatterBottlerBlockEntity
import ru.dbotthepony.mc.otm.block.entity.storage.AbstractStorageImportExport.Companion.FILTER_KEY
import ru.dbotthepony.mc.otm.block.entity.storage.StoragePowerSupplierBlockEntity
import ru.dbotthepony.mc.otm.datagen.modLocation
import ru.dbotthepony.mc.otm.registry.MBlocks
import ru.dbotthepony.mc.otm.registry.MItems
import ru.dbotthepony.mc.otm.registry.MRegistry

fun addLootTables(lootTables: LootTables) {
	lootTables.dropsSelf(MRegistry.DECORATIVE_CRATE.allBlocks.values)

	lootTables.dropsSelf(MRegistry.CARGO_CRATES.allBlocks.values)
	lootTables.dropsSelf(MRegistry.INDUSTRIAL_GLASS.allBlocks.values)
	lootTables.dropsSelf(MRegistry.INDUSTRIAL_GLASS_PANE.allBlocks.values)
	lootTables.dropsSelf(MRegistry.TRITANIUM_BLOCK.allBlocks.values)
	lootTables.dropsSelf(MRegistry.TRITANIUM_WALL.allBlocks.values)
	lootTables.dropsSelf(MRegistry.TRITANIUM_STAIRS.allBlocks.values)
	lootTables.createSlabItemTable(MRegistry.TRITANIUM_SLAB.allBlocks.values)
	lootTables.dropsSelf(MRegistry.VENT.allBlocks.values)
	lootTables.dropsSelf(MRegistry.VENT_ALTERNATIVE.allBlocks.values)
	lootTables.dropsSelf(MRegistry.FLOOR_TILES.blocks.values)
	lootTables.dropsSelf(MRegistry.UNREFINED_FLOOR_TILES.blocks.values)
	lootTables.dropsSelf(MRegistry.TRITANIUM_STRIPED_BLOCK.flatBlocks)
	lootTables.dropsSelf(MRegistry.TRITANIUM_STRIPED_STAIRS.flatBlocks)
	lootTables.dropsSelf(MRegistry.TRITANIUM_STRIPED_WALL.flatBlocks)
	lootTables.createSlabItemTable(MRegistry.TRITANIUM_STRIPED_SLAB.flatBlocks)

	lootTables.dropsSelf(MBlocks.CARBON_FIBRE_BLOCK)
	lootTables.dropsSelf(MBlocks.TRITANIUM_RAW_BLOCK)
	lootTables.dropsSelf(MBlocks.TRITANIUM_STRIPED_BLOCK)
	lootTables.dropsSelf(MBlocks.TRITANIUM_STRIPED_WALL)
	lootTables.dropsSelf(MBlocks.TRITANIUM_STRIPED_STAIRS)
	lootTables.createSlabItemTable(MBlocks.TRITANIUM_STRIPED_SLAB)
	lootTables.dropsSelf(MBlocks.MATTER_CABLE)
	lootTables.dropsSelf(MBlocks.GRAVITATION_STABILIZER)
	lootTables.dropsOther(MBlocks.GRAVITATION_STABILIZER_LENS, MBlocks.GRAVITATION_STABILIZER)

	lootTables.dropsSelf(MBlocks.LABORATORY_LAMP)
	lootTables.dropsSelf(MBlocks.LABORATORY_LAMP_INVERTED)
	lootTables.dropsSelf(MBlocks.DANGER_STRIPE_BLOCK)
	lootTables.dropsSelf(MBlocks.METAL_BEAM)

	lootTables.dropsSelf(MBlocks.TRITANIUM_TRAPDOOR)
	lootTables.dropsSelf(MRegistry.TRITANIUM_PRESSURE_PLATE.allBlocks.values)

	lootTables.blockSimple(MBlocks.PHANTOM_ATTRACTOR) {
		it.add(LootItem.lootTableItem(MBlocks.PHANTOM_ATTRACTOR)
			.`when`(LootItemBlockStatePropertyCondition
				.hasBlockStateProperties(MBlocks.PHANTOM_ATTRACTOR)
				.setProperties(StatePropertiesPredicate.Builder.properties()
					.hasProperty(BlockStateProperties.DOUBLE_BLOCK_HALF, DoubleBlockHalf.LOWER))))
	}

	lootTables.provider(LootContextParamSets.ADVANCEMENT_ENTITY) {
		it.accept(modLocation("research_all_android"), singleLootPool {
			it.add(LootItem.lootTableItem(MItems.EXOSUIT_INVENTORY_UPGRADE_HUGE))
		})

		it.accept(modLocation("tritanium_block3"), singleLootPool {
			it.add(LootItem.lootTableItem(Items.WHITE_DYE).setQuality(8))
			it.add(LootItem.lootTableItem(Items.ORANGE_DYE).setQuality(8))
			it.add(LootItem.lootTableItem(Items.MAGENTA_DYE).setQuality(8))
			it.add(LootItem.lootTableItem(Items.LIGHT_BLUE_DYE).setQuality(8))
			it.add(LootItem.lootTableItem(Items.YELLOW_DYE).setQuality(8))
			it.add(LootItem.lootTableItem(Items.LIME_DYE).setQuality(8))
			it.add(LootItem.lootTableItem(Items.PINK_DYE).setQuality(8))
			it.add(LootItem.lootTableItem(Items.GRAY_DYE).setQuality(8))
			it.add(LootItem.lootTableItem(Items.LIGHT_GRAY_DYE).setQuality(8))
			it.add(LootItem.lootTableItem(Items.CYAN_DYE).setQuality(8))
			it.add(LootItem.lootTableItem(Items.PURPLE_DYE).setQuality(8))
			it.add(LootItem.lootTableItem(Items.BLUE_DYE).setQuality(8))
			it.add(LootItem.lootTableItem(Items.BROWN_DYE).setQuality(8))
			it.add(LootItem.lootTableItem(Items.GREEN_DYE).setQuality(8))
			it.add(LootItem.lootTableItem(Items.RED_DYE).setQuality(8))
			it.add(LootItem.lootTableItem(Items.BLACK_DYE).setQuality(8))
		})

		it.accept(modLocation("tritanium_block4"), singleLootPool {
			it.add(LootItem.lootTableItem(Items.WHITE_DYE).setQuality(64))
			it.add(LootItem.lootTableItem(Items.ORANGE_DYE).setQuality(64))
			it.add(LootItem.lootTableItem(Items.MAGENTA_DYE).setQuality(64))
			it.add(LootItem.lootTableItem(Items.LIGHT_BLUE_DYE).setQuality(64))
			it.add(LootItem.lootTableItem(Items.YELLOW_DYE).setQuality(64))
			it.add(LootItem.lootTableItem(Items.LIME_DYE).setQuality(64))
			it.add(LootItem.lootTableItem(Items.PINK_DYE).setQuality(64))
			it.add(LootItem.lootTableItem(Items.GRAY_DYE).setQuality(64))
			it.add(LootItem.lootTableItem(Items.LIGHT_GRAY_DYE).setQuality(64))
			it.add(LootItem.lootTableItem(Items.CYAN_DYE).setQuality(64))
			it.add(LootItem.lootTableItem(Items.PURPLE_DYE).setQuality(64))
			it.add(LootItem.lootTableItem(Items.BLUE_DYE).setQuality(64))
			it.add(LootItem.lootTableItem(Items.BROWN_DYE).setQuality(64))
			it.add(LootItem.lootTableItem(Items.GREEN_DYE).setQuality(64))
			it.add(LootItem.lootTableItem(Items.RED_DYE).setQuality(64))
			it.add(LootItem.lootTableItem(Items.BLACK_DYE).setQuality(64))
		})
	}

	lootTables.tile(MBlocks.ENERGY_SERVO)

	lootTables.tile(MBlocks.ENERGY_COUNTER,
		EnergyCounterBlockEntity.IO_LIMIT_KEY, EnergyCounterBlockEntity.PASSED_ENERGY_KEY,
		EnergyCounterBlockEntity.POWER_HISTORY_KEY, EnergyCounterBlockEntity.POWER_HISTORY_POINTER_KEY)

	lootTables.tile(MBlocks.CHEMICAL_GENERATOR,
		ChemicalGeneratorBlockEntity.WORK_TICKS_KEY,
		ChemicalGeneratorBlockEntity.WORK_TICKS_TOTAL_KEY,
		ENERGY_KEY,
		REDSTONE_SIGNAL_KEY,
		REDSTONE_SETTING_KEY,
	)

	lootTables.dropsSelf(MBlocks.STORAGE_CABLE)
	lootTables.poweredTile(MBlocks.ANDROID_STATION)
	lootTables.basicTile(MBlocks.BATTERY_BANK)
	lootTables.poweredTile(MBlocks.DRIVE_VIEWER)

	lootTables.poweredTile(MBlocks.STORAGE_BUS, TileNbtCopy(FILTER_KEY))
	lootTables.poweredTile(MBlocks.STORAGE_IMPORTER, TileNbtCopy(FILTER_KEY))
	lootTables.poweredTile(MBlocks.STORAGE_EXPORTER, TileNbtCopy(FILTER_KEY))
	lootTables.poweredTile(MBlocks.STORAGE_POWER_SUPPLIER, TileNbtCopy(StoragePowerSupplierBlockEntity.POWER_PASSED_KEY))
	lootTables.poweredTile(MBlocks.DRIVE_RACK)

	lootTables.matterWorkerTile(MBlocks.MATTER_DECOMPOSER)
	lootTables.matterWorkerTile(MBlocks.MATTER_REPLICATOR)
	lootTables.matterWorkerTile(MBlocks.MATTER_RECYCLER)
	lootTables.workerTile(MBlocks.MATTER_SCANNER)
	lootTables.workerTile(MBlocks.PLATE_PRESS)

	lootTables.basicTile(MBlocks.MATTER_PANEL, TileNbtCopy("tasks"))
	lootTables.basicTile(MBlocks.PATTERN_STORAGE)
	lootTables.basicTile(MBlocks.MATTER_CAPACITOR_BANK)
	lootTables.poweredTile(MBlocks.MATTER_BOTTLER,
		TileNbtCopy(MATTER_STORAGE_KEY), TileNbtCopy(MatterBottlerBlockEntity.IS_BOTTLING_KEY))

	lootTables.block {
		it.accept(MBlocks.TRITANIUM_DOOR.lootTable, BlockLoot.createDoorTable(MBlocks.TRITANIUM_DOOR))
	}
}
